'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _babelPluginSyntaxFlow = require('babel-plugin-syntax-flow');

var _babelPluginSyntaxFlow2 = _interopRequireDefault(_babelPluginSyntaxFlow);

var _babelTypes = require('babel-types');

var t = _interopRequireWildcard(_babelTypes);

var _babelAddFlowComments = require('babel-add-flow-comments');

var _globby = require('globby');

var _globby2 = _interopRequireDefault(_globby);

var _uppercamelcase = require('uppercamelcase');

var _uppercamelcase2 = _interopRequireDefault(_uppercamelcase);

var _s2sUtils = require('s2s-utils');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var createUnion = function createUnion(union) {
  return (0, _s2sUtils.template)('export type Action = UNION')({
    UNION: t.unionTypeAnnotation(union)
  });
};

var createInitAction = (0, _s2sUtils.template)('export type ReduxInitAction = { type: \'@@INIT\' }');

function createActionName(path) {
  return (0, _uppercamelcase2.default)((0, _s2sUtils.getParentDirName)(path)) + 'Action';
}

exports.default = function () {
  return {
    inherits: _babelPluginSyntaxFlow2.default,
    name: 's2s-action-root',
    visitor: {
      Program: {
        exit: function exit(programPath, state) {
          var file = state.file;

          (0, _babelAddFlowComments.removeFlowComment)(file.ast.comments);
          var _state$opts = state.opts,
              input = _state$opts.input,
              output = _state$opts.output;

          var globOptions = Object.assign({ absolute: true }, state.opts.globOptions);

          if (!input) {
            throw new Error('require input option');
          }

          if (!output) {
            throw new Error('require output option');
          }

          var files = _globby2.default.sync(input, globOptions);

          var imports = files.map(function (f) {
            return {
              source: (0, _s2sUtils.getImportPath)(output, f),
              name: createActionName(f)
            };
          }).map(function (_ref) {
            var name = _ref.name,
                source = _ref.source;

            var im = t.importDeclaration([t.importSpecifier(t.identifier(name), t.identifier('Action'))], t.stringLiteral(source));
            // $FlowFixMe
            im.importKind = 'type';
            return im;
          });

          var union = files.map(createActionName).map(function (name) {
            return t.genericTypeAnnotation(t.identifier(name));
          });

          var initAction = 'ReduxInitAction';
          union.unshift(t.genericTypeAnnotation(t.identifier(initAction)));

          var action = createUnion(union);

          programPath.node.body = [].concat(_toConsumableArray(imports), [t.noop(), createInitAction(), t.noop(), action]);

          (0, _babelAddFlowComments.addFlowComment)(programPath);
        }
      }
    }
  };
};